Arguments
- db
A database connection to an m-Path Sense database.
- participant_id
A character string identifying a single participant. Use
get_participantsto retrieve all participants from the database. Leave empty to get data for all participants.- start_date
Optional search window specifying date where to begin search. Must be convertible to date using as.Date. Use first_date to find the date of the first entry for a participant.
- end_date
Optional search window specifying date where to end search. Must be convertible to date using as.Date. Use last_date to find the date of the last entry for a participant.